@@ -94,7 +94,7 @@ class PlexClient(PlexObject):
         self._initpath = self.key
         data = self.query(self.key, timeout=timeout)
         if not data:
-            raise NotFound("Client not found at %s" % self._baseurl)
+            raise NotFound(f"Client not found at {self._baseurl}")
         if self._clientIdentifier:
             client = next(
                 (
@@ -106,8 +106,7 @@ class PlexClient(PlexObject):
             )
             if client is None:
                 raise NotFound(
-                    "Client with identifier %s not found at %s"
-                    % (self._clientIdentifier, self._baseurl)
+                    f"Client with identifier {self._clientIdentifier} not found at {self._baseurl}"
                 )
         else:
             client = data[0]
@@ -131,17 +130,20 @@ class PlexClient(PlexObject):
         self.platformVersion = data.attrib.get('platformVersion')
         self.title = data.attrib.get('title') or data.attrib.get('name')
         # Active session details
-        # Since protocolCapabilities is missing from /sessions we cant really control this player without
+        # Since protocolCapabilities is missing from /sessions we can't really control this player without
         # creating a client manually.
         # Add this in next breaking release.
         # if self._initpath == 'status/sessions':
         self.device = data.attrib.get('device')         # session
+        self.profile = data.attrib.get('profile')       # session
         self.model = data.attrib.get('model')           # session
         self.state = data.attrib.get('state')           # session
         self.vendor = data.attrib.get('vendor')         # session
         self.version = data.attrib.get('version')       # session
-        self.local = utils.cast(bool, data.attrib.get('local', 0))
-        self.address = data.attrib.get('address')        # session
+        self.local = utils.cast(bool, data.attrib.get('local', 0))  # session
+        self.relayed = utils.cast(bool, data.attrib.get('relayed', 0))  # session
+        self.secure = utils.cast(bool, data.attrib.get('secure', 0))  # session
+        self.address = data.attrib.get('address')       # session
         self.remotePublicAddress = data.attrib.get('remotePublicAddress')
         self.userID = data.attrib.get('userID')
 
@@ -183,7 +185,7 @@ class PlexClient(PlexObject):
         if response.status_code not in (200, 201, 204):
             codename = codes.get(response.status_code)[0]
             errtext = response.text.replace('\n', ' ')
-            message = '(%s) %s; %s %s' % (response.status_code, codename, response.url, errtext)
+            message = f'({response.status_code}) {codename}; {response.url} {errtext}'
             if response.status_code == 401:
                 raise Unauthorized(message)
             elif response.status_code == 404:
@@ -210,8 +212,7 @@ class PlexClient(PlexObject):
         controller = command.split('/')[0]
         headers = {'X-Plex-Target-Client-Identifier': self.machineIdentifier}
         if controller not in self.protocolCapabilities:
-            log.debug("Client %s doesn't support %s controller."
-                      "What your trying might not work" % (self.title, controller))
+            log.debug("Client %s doesn't support %s controller. What your trying might not work", self.title, controller)
 
         proxy = self._proxyThroughServer if proxy is None else proxy
         query = self._server.query if proxy else self.query
@@ -225,7 +226,7 @@ class PlexClient(PlexObject):
             self.sendCommand(ClientTimeline.key, wait=0)
 
         params['commandID'] = self._nextCommandId()
-        key = '/player/%s%s' % (command, utils.joinArgs(params))
+        key = f'/player/{command}{utils.joinArgs(params)}'
 
         try:
             return query(key, headers=headers)
@@ -250,8 +251,8 @@ class PlexClient(PlexObject):
             raise BadRequest('PlexClient object missing baseurl.')
         if self._token and (includeToken or self._showSecrets):
             delim = '&' if '?' in key else '?'
-            return '%s%s%sX-Plex-Token=%s' % (self._baseurl, key, delim, self._token)
-        return '%s%s' % (self._baseurl, key)
+            return f'{self._baseurl}{key}{delim}X-Plex-Token={self._token}'
+        return f'{self._baseurl}{key}'
 
     # ---------------------
     # Navigation Commands
@@ -318,21 +319,21 @@ class PlexClient(PlexObject):
             Parameters:
                 media (:class:`~plexapi.media.Media`): Media object to navigate to.
                 **params (dict): Additional GET parameters to include with the command.
-
-            Raises:
-                :exc:`~plexapi.exceptions.Unsupported`: When no PlexServer specified in this object.
         """
-        if not self._server:
-            raise Unsupported('A server must be specified before using this command.')
         server_url = media._server._baseurl.split(':')
-        self.sendCommand('mirror/details', **dict({
-            'machineIdentifier': self._server.machineIdentifier,
+        command = {
+            'machineIdentifier': media._server.machineIdentifier,
             'address': server_url[1].strip('/'),
             'port': server_url[-1],
             'key': media.key,
             'protocol': server_url[0],
-            'token': media._server.createToken()
-        }, **params))
+            **params,
+        }
+        token = media._server.createToken()
+        if token:
+            command["token"] = token
+
+        self.sendCommand("mirror/details", **command)
 
     # -------------------
     # Playback Commands
@@ -488,12 +489,7 @@ class PlexClient(PlexObject):
                     representing the beginning (default 0).
                 **params (dict): Optional additional parameters to include in the playback request. See
                     also: https://github.com/plexinc/plex-media-player/wiki/Remote-control-API#modified-commands
-
-            Raises:
-                :exc:`~plexapi.exceptions.Unsupported`: When no PlexServer specified in this object.
         """
-        if not self._server:
-            raise Unsupported('A server must be specified before using this command.')
         server_url = media._server._baseurl.split(':')
         server_port = server_url[-1].strip('/')
 
@@ -509,19 +505,24 @@ class PlexClient(PlexObject):
         if mediatype == "audio":
             mediatype = "music"
 
-        playqueue = media if isinstance(media, PlayQueue) else self._server.createPlayQueue(media)
-        self.sendCommand('playback/playMedia', **dict({
+        playqueue = media if isinstance(media, PlayQueue) else media._server.createPlayQueue(media)
+        command = {
             'providerIdentifier': 'com.plexapp.plugins.library',
-            'machineIdentifier': self._server.machineIdentifier,
+            'machineIdentifier': media._server.machineIdentifier,
             'protocol': server_url[0],
             'address': server_url[1].strip('/'),
             'port': server_port,
             'offset': offset,
             'key': media.key or playqueue.selectedItem.key,
-            'token': media._server.createToken(),
             'type': mediatype,
-            'containerKey': '/playQueues/%s?window=100&own=1' % playqueue.playQueueID,
-        }, **params))
+            'containerKey': f'/playQueues/{playqueue.playQueueID}?window=100&own=1',
+            **params,
+        }
+        token = media._server.createToken()
+        if token:
+            command["token"] = token
+
+        self.sendCommand("playback/playMedia", **command)
 
     def setParameters(self, volume=None, shuffle=None, repeat=None, mtype=DEFAULT_MTYPE):
         """ Set multiple playback parameters at once.
@@ -620,7 +621,7 @@ class ClientTimeline(PlexObject):
         self.protocol = data.attrib.get('protocol')
         self.providerIdentifier = data.attrib.get('providerIdentifier')
         self.ratingKey = utils.cast(int, data.attrib.get('ratingKey'))
-        self.repeat = utils.cast(bool, data.attrib.get('repeat'))
+        self.repeat = utils.cast(int, data.attrib.get('repeat'))
         self.seekRange = data.attrib.get('seekRange')
         self.shuffle = utils.cast(bool, data.attrib.get('shuffle'))
         self.state = data.attrib.get('state')
